#ifndef noise_h
#define noise_h

/**
 * return perlin noise value at coords (x,y,z)
 *
 *  	From:		Craig McNaughton - view profile
 *  	Date:		Tues, Aug 27 2002 4:13 am
 *  	Email: 		"Craig McNaughton" <c...@pandromeda.com>
 *  	Groups: 		comp.graphics.algorithms
 *  	Not yet rated
 *  	Rating:	 
 *  	show options
 *
 *  	"Marcus Fritzsch" <mCutAwayThisP...@fritschy.de> wrote in message
 *  	news:3D6ACAF8.9000608@fritschy.de...
 *
 *  	> > ?  That would give you the behavior you're seeing.  If so, try something
 *  	> > like:
 *  	> > for (x = 0 to 255)
 *  	> >   for (y = 0 to 255)
 *  	> >      pixel(x, y) = noise(x / 20, y / 20)
 *  	> That works! thx... but, i don't know why it behaves like this?!
 *
 *  	Perlin noise works by calculating a gradient (i.e. slope) at each integer
 *  	lattice point, but assumes the value at each integer point is zero. It uses
 *  	the gradients to calculate values at points inbetween the integer points. So
 *  	between each integer point the noise function will go up or down smoothly to
 *  	an intermediate value, but it'll always be zero at the integers.  This kind
 *  	of noise function is sometimes called "Gradient Noise" for this reason.
 *
 *  	There are other variations of noise functions.  "Value" noise finds a random
 *  	value at each integer lattice point, and assume the gradient is zero.
 *  	"Value-Gradient" noise finds both random values and gradients, and uses both
 *  	to interpolate the values between the integer coordinates.  These different
 *  	types of noise have quite different visual appearances.
 *
 *  	Check out "Texturing and Modelling, A Procedural Approach"  (by Ebert,
 *  	Musgrave, Peachey, Perlin, and Worley.  ISBN 0-12-228730-4) for a detailed
 *  	discussion on (and implementation of) different types of noises, and all the
 *  	cool things you can do with them.
 *
 *  	C.
 *
 *  	-- 
 *  	-
 *  	Craig McNaughton
 *  	Software Engineering,
 *  	Pandromeda.
 *
 * @param x x coordinate
 * @param y y coordinate
 * @param z z coordinate
 */
float fnoise3 (float x, float y, float z);

#endif // noise_h
